@@ -1,322 +1,209 @@
-setMethod(
-	f = "print",
-	signature = signature(x = 'confmirtClass'),
-	definition = function(x, ...)
-	{
-		cat("\nCall:\n", paste(deparse(x@Call), sep = "\n", collapse = "\n"), 
-			"\n\n", sep = "")
-		cat("Full-information item factor analysis with ", ncol(x@Theta), " factors \n", sep="")
-		if(length(x@logLik) > 0){
-			cat("Log-likelihood = ", x@logLik,", SE = ",round(x@SElogLik,3), "\n",sep='')			
-			cat("AIC =", x@AIC, "\n")			
-			cat("BIC =", x@BIC, "\n")
-			if(x@p < 1)
-				cat("G^2 = ", round(x@G2,2), ", df = ", 
-					x@df, ", p = ", round(x@p,4), "\n", sep="")
-			else 
-				cat("G^2 = ", NA, ", df = ", 
-					x@df, ", p = ", NA, "\n", sep="")		
-		}
-		if(x@converge == 1)	
-			cat("Converged in ", x@cycles, " iterations.\n", sep="")
-		else 	
-			cat("Estimation stopped after ", x@cycles, " iterations.\n", sep="")	
-	} 
-)
-
-setMethod(
-	f = "show",
-	signature = signature(object = 'confmirtClass'),
-	definition = function(object)
-	{
-		cat("\nCall:\n", paste(deparse(object@Call), sep = "\n", collapse = "\n"), 
-			"\n\n", sep = "")
-		cat("Full-information item factor analysis with ", ncol(object@Theta), " factors \n", sep="")
-		if(length(object@logLik) > 0){
-			cat("Log-likelihood = ", object@logLik,", SE = ",round(object@SElogLik,3), "\n",sep='')
-			cat("AIC =", object@AIC, "\n")			
-			cat("BIC =", object@BIC, "\n")
-			if(object@p < 1)	
-				cat("G^2 = ", round(object@G2,2), ", df = ", 
-					object@df, ", p = ", round(object@p,4), "\n", sep="")
-			else 
-				cat("G^2 = ", NA, ", df = ", 
-					object@df, ", p = ", NA, "\n", sep="")
-		}
-		if(object@converge == 1)	
-			cat("Converged in ", object@cycles, " iterations.\n", sep="")
-		else 	
-			cat("Estimation stopped after ", object@cycles, " iterations.\n", sep="")	
-	} 
-)
-
-setMethod(
-	f = "summary",
-	signature = 'confmirtClass',
-	definition = function(object, digits = 3, ...)
-	{
-		if(any(object@estComp)) stop('No factor metric for noncompensatory models')
-		nfact <- ncol(object@F)		
-		F <- object@F		
-		colnames(F) <- paste("F_", 1:ncol(F),sep="")						
-		SS <- apply(F^2,2,sum)			
-		cat("\nFactor loadings metric: \n")
-		print(cbind(F),digits)		
-		cat("\nSS loadings: ",round(SS,digits), "\n")		
-		cat("\nFactor correlations: \n")
-		Phi <- cov2cor(object@gpars$sig)	  
-		Phi <- round(Phi, digits)
-		colnames(Phi) <- rownames(Phi) <- colnames(F)
-		print(Phi)				
-		invisible(F)  	  
-	}
-)
-
-setMethod(
-	f = "coef",
-	signature = 'confmirtClass',
-	definition = function(object, SE = TRUE, print.gmeans = FALSE, digits = 3, ...)
-	{  
-		nfact <- ncol(object@Theta)	
-		a <- matrix(object@pars[ ,1:nfact],ncol=nfact)
-		d <- matrix(object@pars[,(nfact+1):ncol(object@pars)],
-			ncol = ncol(object@pars)-nfact)    	
-
-		parameters <- cbind(object@pars,object@guess)
-		SEs <- cbind(object@SEpars,object@SEg)	
-		colnames(SEs) <- colnames(parameters) <- c(paste("a_",1:nfact,sep=""),
-			paste("d_",1:(ncol(object@pars)-nfact),sep=""),"guess")					
-		cat("\nITEM PARAMTERS: \n")
-		print(parameters, digits)
-		if(SE){
-			cat("\nStd. Errors: \n")	
-			print(SEs, digits)
-		}	
-		u <- object@gpars$u	
-		sig <- object@gpars$sig	
-		cat("\nGROUP PARAMETERS: \n")
-		if(print.gmeans){
-			cat("Means: \n")
-			print(u,digits)
-			cat("\nStd. Errors: \n")	
-			print(object@SEgpars$SEu, digits)	
-		}
-		cat("Covariance: \n")
-		print(sig,digits)
-		if(SE){
-			cat("\nStd. Errors: \n")	
-			print(object@SEgpars$SEsig, digits)	
-		}
-		invisible(list(pars = parameters,mu = u,sigma = sig))	
-	}
-)
-
-setMethod(
-	f = "residuals",
-	signature = signature(object = 'confmirtClass'),
-	definition = function(object, restype = 'LD', digits = 3, ...)
-	{ 
-		fulldata <- object@fulldata	
-		data <- object@data
-		data[data==99] <- NA		
-		N <- nrow(data)
-		K <- object@K
-		J <- length(K)
-		sig <- object@gpars$sig	
-		nfact <- ncol(object@F)
-		theta <- seq(-4,4, length.out = round(20/nfact))
-		Theta <- thetaComb(theta,nfact)		
-		lambdas <- matrix(object@pars[,1:nfact], J)
-		lambdas[is.na(lambdas)] <- 0
-		zetas <- as.vector(t(object@pars[,(nfact+1):ncol(object@pars)]))
-		zetas <- na.omit(zetas)
-		zetalist <- list()
-		loc <- 1
-		for(i in 1:J){
-			zetalist[[i]] <- zetas[loc:(loc+K[i]-2)]
-			loc <- loc + K[i] - 1		
-		}
-		guess <- object@guess
-		guess[is.na(guess)] <- 0	
-		Ksums <- cumsum(K) - 1	
-		itemloc <- object@itemloc
-		res <- matrix(0,J,J)
-		diag(res) <- NA
-		colnames(res) <- rownames(res) <- colnames(data)
-		prior <- dmvnorm(Theta,rep(0,nfact),sig)
-		prior <- prior/sum(prior)		
-		if(restype == 'LD'){	
-			for(i in 1:J){				
-				for(j in 1:J){			
-					if(i < j){
-						if(K[i] > 2) P1 <- P.poly(lambdas[i,],zetalist[[i]],Theta,itemexp=TRUE)
-						else { 
-							P1 <- P.mirt(lambdas[i,],zetalist[[i]], Theta, guess[i])
-							P1 <- cbind(1 - P1, P1)
-						}	
-						if(K[j] > 2) P2 <- P.poly(lambdas[j,],zetalist[[j]],Theta,itemexp=TRUE)
-						else {
-							P2 <- P.mirt(lambdas[j,],zetalist[[j]], Theta, guess[j])	
-							P2 <- cbind(1 - P2, P2)
-						}
-						tab <- table(data[,i],data[,j])		
-						Etab <- matrix(0,K[i],K[j])
-						for(k in 1:K[i])
-							for(m in 1:K[j])						
-								Etab[k,m] <- N * sum(P1[,k] * P2[,m] * prior)	
-						s <- gamma.cor(tab) - gamma.cor(Etab)
-						if(s == 0) s <- 1				
-						res[j,i] <- sum(((tab - Etab)^2)/Etab) * sign(s)
-						res[i,j] <- sqrt(abs(res[j,i]) / (N * min(c(K[i],K[j]) - 1))) 					
-					}					
-				}
-			}		
-			cat("LD matrix:\n\n")	
-			res <- round(res,digits)    	
-			return(res)
-		}
-		if(restype == 'exp'){
-			if(length(object@tabdata) == 0) stop('Expected response vectors cannot be computed because logLik() 
-				has not been run or the data contains missing responses.')
-			tabdata <- object@tabdata
-			res <- (tabdata[,J+1] - tabdata[,J+2]) / sqrt(tabdata[,J+2])
-			tabdata <- round(cbind(tabdata,res),digits)
-			colnames(tabdata) <- c(colnames(object@data), 'freq', 'exp', 'std_res')
-			return(tabdata)
-		}
-	}
-)
-
-setMethod(
-	f = "logLik",
-	signature = signature(object = 'confmirtClass'),
-	definition = function(object, draws = 2000, G2 = TRUE)
-	{	
-		nfact <- ncol(object@Theta)
-		N <- nrow(object@Theta)
-		J <- length(object@K)
-		pars <- object@pars
-		lambdas <- pars[,1:nfact]
-		lambdas[is.na(lambdas)] <- 0
-		zetas <- pars[,(nfact+1):ncol(pars)]
-		zetas <- t(zetas)[!is.na(t(zetas))]		
-		mu <- object@gpars$u
-		sigma <- object@gpars$sig		
-		LL <- matrix(0,N,draws)		
-		guess <- object@guess
-		guess[is.na(guess)] <- 0
-		K <- object@K	
-		fulldata <- object@fulldata	
-		for(i in 1:draws){
-			theta <- rmvnorm(N,mu,sigma)				
-			LL[,i] <- .Call('logLik', 					
-						as.numeric(lambdas),
-						as.numeric(zetas),
-						as.numeric(guess),
-						as.numeric(theta),
-						as.integer(fulldata),
-						as.integer(object@itemloc-1),
-						as.integer(object@K),
-						as.integer(J),
-						as.integer(N),
-						as.integer(nfact),
-						as.integer(object@estComp))		
-		}		
-		rwmeans <- rowMeans(LL)
-		logLik <- sum(log(rwmeans))				
-		pats <- apply(fulldata,1,paste,collapse = "/")
-		freqs <- table(pats)
-		nfreqs <- length(freqs)		
-		r <- as.vector(freqs)
-		ncolfull <- ncol(fulldata)
-		tabdata <- unlist(strsplit(cbind(names(freqs)),"/"))
-		tabdata <- matrix(as.numeric(tabdata),nfreqs,ncolfull,TRUE)
-		tabdata <- cbind(tabdata,r)		 		 				
-		pats <- apply(fulldata,1,paste,collapse = "/")
-		freqs <- table(pats)			
-		r <- as.vector(freqs)
-		logN <- 0
-		logr <- rep(0,length(r))
-		for (i in 1:N) logN <- logN + log(i)
-		for (i in 1:length(r)) 
-			for (j in 1:r[i]) 
-				logr[i] <- logr[i] + log(j)    		
-		if(sum(logr) != 0)		
-			logLik <- logLik + logN/sum(logr)			
-		SElogLik <- sqrt(var(log(rowMeans(LL))) / draws)
-		x <- object@estpars	
-		df <- as.integer(length(r) - sum(x$estlam) - sum(x$estgcov) - 
-			sum(x$estgmeans) - length(zetas) + object@nconstvalues + 
-			nfact*(nfact - 1)/2 - sum(x$estGuess) - 1)			
-		AIC <- (-2) * logLik + 2 * (length(r) - df - 1)
-		BIC <- (-2) * logLik + (length(r) - df - 1)*log(N)
-		if(G2){			
-			data <- object@data
-			if(any(is.na(data))){
-				object@G2 <- 0	
-				object@p <- 1					
-			} else {			
-				pats <- apply(data,1,paste,collapse = "/")			
-				freqs <- table(pats)
-				nfreqs <- length(freqs)		
-				r <- as.vector(freqs)
-				ncolfull <- ncol(data)
-				tabdata <- unlist(strsplit(cbind(names(freqs)),"/"))
-				tabdata <- matrix(as.numeric(tabdata),nfreqs,ncolfull,TRUE)
-				tabdata <- cbind(tabdata,r)	
-				expected <- rep(0,nrow(tabdata))	
-				for (j in 1:nrow(tabdata)){          
-					TFvec <- colSums(ifelse(t(data) == tabdata[j,1:ncolfull],1,0)) == ncolfull        
-					expected[j] <- mean(rwmeans[TFvec])
-					rwmeans[TFvec] <- rwmeans[TFvec]/r[j]
-				}
-				tabdata <- cbind(tabdata,expected*N)
-				G2 <- 2 * sum(log(1/(N*rwmeans)))
-				p <- 1 - pchisq(G2,df) 
-				object@G2 <- G2	
-				object@p <- p
-				object@tabdata <- tabdata
-			}	
-		}	
-		object@logLik <- logLik
-		object@SElogLik <- SElogLik		
-		object@AIC <- AIC
-		object@BIC <- BIC
-		object@df <- df		
-		return(object)
-	} 	
-)
-
-setMethod(
-	f = "anova",
-	signature = signature(object = 'confmirtClass'),
-	definition = function(object, object2, ...)
-	{
-		dots <- list(...)				
-		nitems <- length(object@K)
-		if(length(object@df) == 0 || length(object2@df) == 0) 
-			stop('Use \'logLik\' to obtain likelihood values') 	
-		df <- object@df - object2@df
-		if(df < 0){
-			df <- abs(df)
-			tmp <- object
-			object <- object2
-			object2 <- tmp
-		}
-		X2 <- 2*object2@logLik - 2*object@logLik 
-		AICdiff <- object@AIC - object2@AIC  
-		BICdiff <- object@BIC - object2@BIC  
-		se <- round(object@SElogLik + object2@SElogLik,3)		
-		cat("\nChi-squared difference: \n\nX2 = ", round(X2,3), 
-			" (SE = ",se,"), df = ", df, ", p = ", round(1 - pchisq(X2,df),4), "\n", sep="")
-		cat("AIC difference = ", round(AICdiff,3)," (SE = ", se,")\n", sep='')
-		cat("BIC difference = ", round(BICdiff,3)," (SE = ", se,")\n", sep='')
-	}		
-)
-
-####################
-#Main Function
+# Class "confmirtClass"
+# 
+# Defines the object returned from \code{\link{confmirt}}.
+# 
+# 
+# @name confmirtClass-class
+# @aliases confmirtClass-class coef,confmirtClass-method
+# print,confmirtClass-method residuals,confmirtClass-method
+# show,confmirtClass-method summary,confmirtClass-method
+# anova,confmirtClass-method
+# @docType class
+# @section Objects from the Class: Objects can be created by calls of the form
+# \code{new("confmirtClass", ...)}.
+# @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
+#' @exportClass confmirtClass
+# @keywords classes
+setClass(
+	Class = 'confmirtClass',
+	representation = representation(pars = 'matrix', guess = 'numeric', SEpars = 'matrix', 
+		SEg = 'numeric', gpars = 'list', SEgpars = 'list', estpars = 'list',cycles = 'numeric', 
+		Theta = 'matrix', fulldata = 'matrix', data = 'matrix', K = 'numeric', itemloc = 'numeric',
+		h2 = 'numeric',F = 'matrix', converge = 'numeric', logLik = 'numeric',SElogLik = 'numeric',
+		df = 'integer', AIC = 'numeric', nconstvalues = 'integer', G2 = 'numeric', p = 'numeric',
+		tabdata = 'matrix', BIC = 'numeric', estComp = 'logical', Call = 'call'),	
+	validity = function(object) return(TRUE)
+)	
 
+#' Confirmatory Full-Information Item Factor Analysis for Mixed Data Formats
+#' 
+#' \code{confmirt} fits a conditional (i.e., confirmatory) full-information
+#' maximum-likelihood factor analysis model to dichotomous and polychotomous
+#' data under the item response theory paradigm using Cai's (2010)
+#' Metropolis-Hastings Robbins-Monro algorithm. If requested, lower asymptote
+#' parameters are estimated with a beta prior and are included automatically.
+#' 
+#' 
+#' \code{confmirt} follows a confirmatory item factor analysis strategy that
+#' uses a stochastic version of maximum likelihood estimation described by Cai
+#' (2010). The general equation used for multidimensional item response theory
+#' in this function is in the logistic form with a scaling correction of 1.702.
+#' This correction is applied to allow comparison to mainstream programs such
+#' as TESTFACT (2003) and POLYFACT. Missing data are treated as 'missing at
+#' random' so that each response vector is included in the estimation (i.e.,
+#' full-information). Residuals are computed using the LD statistic (Chen &
+#' Thissen, 1997) in the lower diagonal of the matrix returned by
+#' \code{residuals}, and Cramer's V above the diagonal. For computing the
+#' log-likelihood more accurately see \code{\link{logLik}}.
+#' 
+#' Specification of the confirmatory item factor analysis model follows many of
+#' the rules in the SEM framework for confirmatory factor analysis. The
+#' variances of the latent factors are automatically fixed to 1 to help
+#' facilitate model identification. All parameters may be fixed to constant
+#' values or set equal to other parameters using the appropriate declarations.
+#' Guessing parameters may be specified for dichotomous items and are estimated
+#' with beta priors automatically, and if a guessing parameter is declared for
+#' a polychotomous item it is ignored.
+#' 
+#' \code{coef} displays the item parameters with their associated standard
+#' errors, while use of \code{summary} transforms the slopes into a factor
+#' loadings metric. Also, nested models may be compared by using the
+#' \code{anova} function, where a Chi-squared difference test and AIC/BIC
+#' difference values are displayed.
+#' 
+#' @aliases confmirt coef,confmirt-method summary,confmirt-method
+#' residuals,confmirt-method anova,confmirt-method fitted,confmirt-method
+#' @param data a \code{matrix} or \code{data.frame} that consists of
+#' numerically ordered data
+#' @param model an object returned from \code{confmirt.model()} declarating how
+#' the factor model is to be estimated. See \code{\link{confmirt.model}} for
+#' more details
+#' @param guess fixed values for the pseudo-guessing parameter. Can be entered
+#' as a single value to assign a global guessing parameter or may be entered as
+#' a numeric vector for each item
+#' @param estGuess a logical vector indicating which lower-asymptote parameters
+#' to be estimated (default is null, and therefore is contingent on the values
+#' in \code{guess}). By default, if any value in \code{guess} is greater than 0
+#' then its respective \code{estGuess} value is set to \code{TRUE}.
+#' Additionally, beta priors are automatically imposed for estimated parameters
+#' which correspond to the input guessing values.
+#' @param ncycles the maximum number of MH-RM iterations to be performed. Default is 
+#' 2000
+#' @param printvalue a numeric value to be specified when using the \code{res='exp'}
+#' option. Only prints patterns that have standardized residuals greater than 
+#' \code{abs(printvalue)}. The default (NULL) prints all response patterns
+#' @param burnin number of burn-in cycles to perform before beginning the SEM
+#' stage. Default is 150
+#' @param SEM.cycles number of stochastic EM cycles to perform and average over
+#' before beginning the MH-RM algorithm. Default is 50
+#' @param kdraws number of Metropolis-Hastings imputations of the factor scores
+#' at each iteration. Default is 1
+#' @param tol tolerance that can be reached to terminate the model estimation;
+#' must be achieved on 3 consecutive iterations
+#' @param printcycles logical; display iteration history during estimation?
+#' @param calcLL logical; calculate the log-likelihood via Monte Carlo
+#' integration?
+#' @param draws the number of Monte Carlo draws to estimate the log-likelihood
+#' @param restype type of residuals to be displayed. Can be either \code{'LD'}
+#' for a local dependence matrix (Chen & Thissen, 1997) or \code{'exp'} for the
+#' expected values for the frequencies of every response pattern
+#' @param returnindex logical; return the list containing the item paramter
+#' locations? To be used when specifying prior parameter distributions
+#' @param debug logical; turn on debugging features?
+#' @param object an object of class \code{confmirt}
+#' @param object2 an object of class \code{confmirt}
+#' @param SE logical; print standard errors?
+#' @param print.gmeans logical; print latent factor means?
+#' @param digits the number of significant digits to be rounded
+#' @param technical list specifying subtle parameters that can be adjusted
+#' @param ... additional arguments to be passed
+#' @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
+#' @seealso
+#' \code{\link{expand.table}}, \code{\link{key2binary}}, \code{\link{simdata}},
+#' \code{\link{fscores}}, \code{\link{confmirt.model}}
+#' @references
+#' 
+#' Cai, L. (2010). Metropolis-Hastings Robbins-Monro algorithm for confirmatory
+#' item factor analysis. \emph{Journal of Educational and Behavioral
+#' Statistics, 35}, 307-335.
+#' 
+#' Wood, R., Wilson, D. T., Gibbons, R. D., Schilling, S. G., Muraki, E., &
+#' Bock, R. D. (2003). TESTFACT 4 for Windows: Test Scoring, Item Statistics,
+#' and Full-information Item Factor Analysis [Computer software]. Lincolnwood,
+#' IL: Scientific Software International.
+#' @keywords models
+#' @usage 
+#' confmirt(data, model, guess = 0, estGuess = NULL, ncycles = 2000, burnin = 150, 
+#'   SEM.cycles = 50, kdraws = 1, tol = .001, printcycles = TRUE, calcLL = TRUE, 
+#'   draws = 2000, returnindex = FALSE, debug = FALSE, technical = list(), ...)
+#' 
+#' \S4method{coef}{confmirt}(object, SE = TRUE, print.gmeans = FALSE, digits = 3, ...)
+#' 
+#' \S4method{summary}{confmirt}(object, digits = 3, ...)
+#' 
+#' \S4method{residuals}{confmirt}(object, restype = 'LD', digits = 3, printvalue = NULL, ...)
+#' 
+#' \S4method{anova}{confmirt}(object, object2, ...)
+#'
+#' \S4method{fitted}{confmirt}(object, digits = 3, ...)
+#'
+#' @export confmirt
+#' @examples
+#'  
+#' \dontrun{
+#' #simulate data
+#' a <- matrix(c(
+#' 1.5,NA,
+#' 0.5,NA,
+#' 1.0,NA,
+#' 1.0,0.5,
+#'  NA,1.5,
+#'  NA,0.5,
+#'  NA,1.0,
+#'  NA,1.0),ncol=2,byrow=TRUE)
+#' 
+#' d <- matrix(c(
+#' -1.0,NA,NA,
+#' -1.5,NA,NA,
+#'  1.5,NA,NA,
+#'  0.0,NA,NA,
+#' 3.0,2.0,-0.5,
+#' 2.5,1.0,-1,
+#' 2.0,0.0,NA,
+#' 1.0,NA,NA),ncol=3,byrow=TRUE)
+#' 
+#' sigma <- diag(2)
+#' sigma[1,2] <- sigma[2,1] <- .4
+#' dataset <- simdata(a,d,2000,sigma)
+#' 
+#' #analyses
+#' #CIFA for 2 factor crossed structure
+#' 
+#' model.1 <- confmirt.model()
+#'   F1 = 1-4
+#'   F2 = 4-8
+#'   COV = F1*F2
+#'   
+#' mod1 <- confmirt(dataset,model.1)
+#' coef(mod1)
+#' summary(mod1)
+#' residuals(mod1)
+#' 
+#' #fix first slope at 1.5, and set slopes 7 & 8 to be equal
+#' model.2 <- confmirt.model()
+#'   F1 = 1-4
+#'   F2 = 4-8
+#'   COV = F1*F2
+#'   SLOPE = F1@@1 eq 1.5, F2@@7 eq F2@@8
+#'   
+#' mod2 <- confmirt(dataset,model.2)
+#' anova(mod2,mod1)
+#' 
+#' #####
+#' #bifactor 	
+#' model.3 <- confmirt.model()
+#'   G = 1-8
+#'   F1 = 1-4
+#'   F2 = 5-8
+#' 	
+#' mod3 <- confmirt(dataset,model.3)
+#' coef(mod3)
+#' summary(mod3)
+#' residuals(mod3)
+#' anova(mod1,mod3)
+#' 
+#' }
+#' 
 confmirt <- function(data, model, guess = 0, estGuess = NULL, ncycles = 2000, 
 	burnin = 150, SEM.cycles = 50, kdraws = 1, tol = .001, printcycles = TRUE, 
 	calcLL = TRUE, draws = 2000, returnindex = FALSE, debug = FALSE, technical = list(), ...)
@@ -324,6 +211,8 @@ confmirt <- function(data, model, guess = 0, estGuess = NULL, ncycles = 2000,
 	Call <- match.call()   
 	set.seed(12345)
 	if(!is.null(technical$set.seed)) set.seed(technical$set.seed)
+	ifelse(!is.null(technical$guess.prior.n), guess.prior.n <- technical$guess.prior.n,
+		guess.prior.n <- 20)
 	itemnames <- colnames(data)
 	keywords <- c('SLOPE','INT','COV','MEAN','PARTCOMP','PRIOR')
 	data <- as.matrix(data)		
@@ -631,8 +520,8 @@ confmirt <- function(data, model, guess = 0, estGuess = NULL, ncycles = 2000,
 	if(sum(estGuess) > 0){
 		for(i in 1:J){
 			if(estGuess[i]){
-				a <- guess[i] * 20
-				b <- (1 - guess[i]) * 20
+				a <- guess[i] * guess.prior.n
+				b <- (1 - guess[i]) * guess.prior.n
 				parpriors[[parpriorscount]] <- c(2,guessind[i],a,b)						
 				parpriorscount <- parpriorscount + 1			
 			}
@@ -910,6 +799,7 @@ confmirt <- function(data, model, guess = 0, estGuess = NULL, ncycles = 2000,
 		info <- info + gamma*(Tau - phi %*% t(phi) - info)		
 	} ###END BIG LOOP
 	
+	if(!is.null(technical$return.verbose)) {}
 	cat("\n\n")
 	SEtmp <- diag(solve(info))		
 	if(any(SEtmp < 0)){
@@ -989,6 +879,7 @@ confmirt <- function(data, model, guess = 0, estGuess = NULL, ncycles = 2000,
 	F <- as.matrix(pars[ ,1:nfact]/norm)
 	F[is.na(F)] <- 0		
 	h2 <- rowSums(F^2)
+	names(h2) <- itemnames
 
 	mod <- new('confmirtClass', pars=pars, guess=guess, SEpars=SEpars, SEg = SEg, 
 		gpars=gpars, SEgpars=SEgpars, estpars=estpars,cycles=cycles - SEM.cycles 
@@ -1001,5 +892,315 @@ confmirt <- function(data, model, guess = 0, estGuess = NULL, ncycles = 2000,
 		mod <- logLik(mod,draws)		
 	}	
 	return(mod)
-}	
+}
 
+# Methods
+setMethod(
+	f = "print",
+	signature = signature(x = 'confmirtClass'),
+	definition = function(x, ...)
+	{
+		cat("\nCall:\n", paste(deparse(x@Call), sep = "\n", collapse = "\n"), 
+			"\n\n", sep = "")
+		cat("Full-information item factor analysis with ", ncol(x@Theta), " factors \n", sep="")
+		if(length(x@logLik) > 0){
+			cat("Log-likelihood = ", x@logLik,", SE = ",round(x@SElogLik,3), "\n",sep='')			
+			cat("AIC =", x@AIC, "\n")			
+			cat("BIC =", x@BIC, "\n")
+			if(x@p < 1)
+				cat("G^2 = ", round(x@G2,2), ", df = ", 
+					x@df, ", p = ", round(x@p,4), "\n", sep="")
+			else 
+				cat("G^2 = ", NA, ", df = ", 
+					x@df, ", p = ", NA, "\n", sep="")		
+		}
+		if(x@converge == 1)	
+			cat("Converged in ", x@cycles, " iterations.\n", sep="")
+		else 	
+			cat("Estimation stopped after ", x@cycles, " iterations.\n", sep="")	
+	} 
+)
+
+setMethod(
+	f = "show",
+	signature = signature(object = 'confmirtClass'),
+	definition = function(object)
+	{
+		cat("\nCall:\n", paste(deparse(object@Call), sep = "\n", collapse = "\n"), 
+			"\n\n", sep = "")
+		cat("Full-information item factor analysis with ", ncol(object@Theta), " factors \n", sep="")
+		if(length(object@logLik) > 0){
+			cat("Log-likelihood = ", object@logLik,", SE = ",round(object@SElogLik,3), "\n",sep='')
+			cat("AIC =", object@AIC, "\n")			
+			cat("BIC =", object@BIC, "\n")
+			if(object@p < 1)	
+				cat("G^2 = ", round(object@G2,2), ", df = ", 
+					object@df, ", p = ", round(object@p,4), "\n", sep="")
+			else 
+				cat("G^2 = ", NA, ", df = ", 
+					object@df, ", p = ", NA, "\n", sep="")
+		}
+		if(object@converge == 1)	
+			cat("Converged in ", object@cycles, " iterations.\n", sep="")
+		else 	
+			cat("Estimation stopped after ", object@cycles, " iterations.\n", sep="")	
+	} 
+)
+
+setMethod(
+	f = "summary",
+	signature = 'confmirtClass',
+	definition = function(object, digits = 3, ...)
+	{
+		if(any(object@estComp)) stop('No factor metric for noncompensatory models')
+		nfact <- ncol(object@F)
+		itemnames <- names(object@h2)	
+		F <- object@F
+		rownames(F) <- itemnames
+		colnames(F) <- paste("F_", 1:ncol(F),sep="")						
+		SS <- apply(F^2,2,sum)			
+		cat("\nFactor loadings metric: \n")
+		print(cbind(F),digits)		
+		cat("\nSS loadings: ",round(SS,digits), "\n")		
+		cat("\nFactor correlations: \n")
+		Phi <- cov2cor(object@gpars$sig)	  
+		Phi <- round(Phi, digits)
+		colnames(Phi) <- rownames(Phi) <- colnames(F)
+		print(Phi)				
+		invisible(F)  	  
+	}
+)
+
+setMethod(
+	f = "coef",
+	signature = 'confmirtClass',
+	definition = function(object, SE = TRUE, print.gmeans = FALSE, digits = 3, ...)
+	{  
+		nfact <- ncol(object@Theta)	
+		itemnames <- names(object@h2)
+		a <- matrix(object@pars[ ,1:nfact],ncol=nfact)
+		d <- matrix(object@pars[,(nfact+1):ncol(object@pars)],
+			ncol = ncol(object@pars)-nfact)    	
+
+		parameters <- cbind(object@pars,object@guess)
+		SEs <- cbind(object@SEpars,object@SEg)
+		rownames(parameters) <- itemnames
+		rownames(SEs) <- itemnames
+		colnames(SEs) <- colnames(parameters) <- c(paste("a_",1:nfact,sep=""),
+			paste("d_",1:(ncol(object@pars)-nfact),sep=""),"guess")					
+		cat("\nITEM PARAMETERS: \n")
+		print(parameters, digits)
+		if(SE){
+			cat("\nStd. Errors: \n")	
+			print(SEs, digits)
+		}	
+		u <- object@gpars$u	
+		sig <- object@gpars$sig
+		names(u) <- colnames(sig) <- rownames(sig) <- paste("a_",1:nfact,sep="")	
+		cat("\nGROUP PARAMETERS: \n")
+		if(print.gmeans){
+			cat("Means: \n")
+			print(u,digits)
+			cat("\nStd. Errors: \n")
+			SEu <- object@SEgpars$SEu
+			names(SEu) <- names(u) 	
+			print(SEu, digits)	
+		}
+		cat("Covariance: \n")
+		print(sig,digits)
+		if(SE){
+			cat("\nStd. Errors: \n")
+			SEsig <- object@SEgpars$SEsig
+			colnames(SEsig) <- rownames(SEsig) <- paste("a_",1:nfact,sep="")	
+			print(SEsig, digits)	
+		}
+		invisible(list(pars = parameters,mu = u,sigma = sig))	
+	}
+)
+
+setMethod(
+	f = "residuals",
+	signature = signature(object = 'confmirtClass'),
+	definition = function(object, restype = 'LD', digits = 3, printvalue = NULL, ...)
+	{ 
+		fulldata <- object@fulldata	
+		data <- object@data
+		data[data==99] <- NA		
+		N <- nrow(data)
+		K <- object@K
+		J <- length(K)
+		sig <- object@gpars$sig	
+		nfact <- ncol(object@F)
+		theta <- seq(-4,4, length.out = round(20/nfact))
+		Theta <- thetaComb(theta,nfact)		
+		lambdas <- matrix(object@pars[,1:nfact], J)
+		lambdas[is.na(lambdas)] <- 0
+		zetas <- as.vector(t(object@pars[,(nfact+1):ncol(object@pars)]))
+		zetas <- na.omit(zetas)
+		zetalist <- list()
+		loc <- 1
+		for(i in 1:J){
+			zetalist[[i]] <- zetas[loc:(loc+K[i]-2)]
+			loc <- loc + K[i] - 1		
+		}
+		guess <- object@guess
+		guess[is.na(guess)] <- 0	
+		Ksums <- cumsum(K) - 1	
+		itemloc <- object@itemloc
+		res <- matrix(0,J,J)
+		diag(res) <- NA
+		colnames(res) <- rownames(res) <- colnames(data)
+		prior <- dmvnorm(Theta,rep(0,nfact),sig)
+		prior <- prior/sum(prior)		
+		if(restype == 'LD'){	
+			for(i in 1:J){				
+				for(j in 1:J){			
+					if(i < j){
+						if(K[i] > 2) P1 <- P.poly(lambdas[i,],zetalist[[i]],Theta,itemexp=TRUE)
+						else { 
+							P1 <- P.mirt(lambdas[i,],zetalist[[i]], Theta, guess[i])
+							P1 <- cbind(1 - P1, P1)
+						}	
+						if(K[j] > 2) P2 <- P.poly(lambdas[j,],zetalist[[j]],Theta,itemexp=TRUE)
+						else {
+							P2 <- P.mirt(lambdas[j,],zetalist[[j]], Theta, guess[j])	
+							P2 <- cbind(1 - P2, P2)
+						}
+						tab <- table(data[,i],data[,j])		
+						Etab <- matrix(0,K[i],K[j])
+						for(k in 1:K[i])
+							for(m in 1:K[j])						
+								Etab[k,m] <- N * sum(P1[,k] * P2[,m] * prior)	
+						s <- gamma.cor(tab) - gamma.cor(Etab)
+						if(s == 0) s <- 1				
+						res[j,i] <- sum(((tab - Etab)^2)/Etab) * sign(s)
+						res[i,j] <- sqrt(abs(res[j,i]) / (N * min(c(K[i],K[j]) - 1))) 					
+					}					
+				}
+			}		
+			cat("LD matrix:\n\n")	
+			res <- round(res,digits)    	
+			return(res)
+		}
+		if(restype == 'exp'){
+			if(length(object@tabdata) == 0) stop('Expected response vectors cannot be computed because logLik() 
+				has not been run or the data contains missing responses.')
+			tabdata <- object@tabdata
+			res <- (tabdata[,J+1] - tabdata[,J+2]) / sqrt(tabdata[,J+2])
+			tabdata <- round(cbind(tabdata,res),digits)
+			colnames(tabdata) <- c(colnames(object@data), 'freq', 'exp', 'std_res')
+			if(!is.null(printvalue)){
+				if(!is.numeric(printvalue)) stop('printvalue is not a number.')
+				tabdata <- tabdata[abs(tabdata[ ,ncol(tabdata)]) > printvalue, ]
+			}
+			return(tabdata)
+		}
+	}
+)
+
+setMethod(
+	f = "anova",
+	signature = signature(object = 'confmirtClass'),
+	definition = function(object, object2, ...)
+	{
+		dots <- list(...)				
+		nitems <- length(object@K)
+		if(length(object@df) == 0 || length(object2@df) == 0) 
+			stop('Use \'logLik\' to obtain likelihood values') 	
+		df <- object@df - object2@df
+		if(df < 0){			
+			tmp <- object
+			object <- object2
+			object2 <- tmp
+		}
+		X2 <- 2*object2@logLik - 2*object@logLik 
+		AICdiff <- object@AIC - object2@AIC  
+		BICdiff <- object@BIC - object2@BIC  
+		se <- round(object@SElogLik + object2@SElogLik,3)
+		cat("\nChi-squared difference: \n\nX2 = ", round(X2,3), 
+			" (SE = ",se,"), df = ", df, ", p = ", round(1 - pchisq(X2,abs(df)),4), 
+			"\n", sep="")
+		cat("AIC difference = ", round(AICdiff,3)," (SE = ", se,")\n", sep='')
+		cat("BIC difference = ", round(BICdiff,3)," (SE = ", se,")\n", sep='')
+	}		
+)
+
+#' @rdname fscores-methods  	
+setMethod(
+	f = "fscores",
+	signature = 'confmirtClass',
+	definition = function(object, full.scores = FALSE, ndraws = 3000, thin = 5, ...)
+	{ 	
+		cand.t.var <- 1
+		estComp <- object@estComp
+		sig <- object@gpars$sig
+		mu <- object@gpars$u
+		theta0 <- object@Theta
+		K <- object@K
+		nfact <- ncol(theta0)
+		lambdas <- matrix(object@pars[,1:nfact],ncol=nfact)
+		lambdas[is.na(lambdas)] <- 0
+		zetas <- na.omit(as.numeric(t(object@pars[,(nfact+1):ncol(object@pars)])))
+		guess <- object@guess
+		guess[is.na(guess)] <- 0
+		data <- cbind(object@data,object@fulldata)		
+		Names <- c(colnames(object@data[,1:length(K)]),paste("F",1:nfact,sep=''),paste("SE_F",1:nfact,sep=''))
+		tabdata <- unique(data)[,-c(1:length(K))]			
+		itemloc <- object@itemloc
+		Theta <- list()
+		for(i in 1:nfact)
+			Theta[[i]] <- matrix(0,ncol=ndraws/thin,nrow=nrow(tabdata))		
+		theta0 <- matrix(0,nrow(tabdata),nfact)
+		for(i in 1:30){			
+			theta0 <- draw.thetas(theta0,lambdas,zetas,guess,tabdata,K,itemloc,cand.t.var,sig,mu,estComp)
+			if(attr(theta0,'Proportion Accepted') > .4) cand.t.var <- cand.t.var + .2
+			if(attr(theta0,'Proportion Accepted') < .3) cand.t.var <- cand.t.var - .2
+		}
+		ind <- 1
+		for(i in 1:ndraws){			
+			theta0 <- draw.thetas(theta0,lambdas,zetas,guess,tabdata,K,itemloc,cand.t.var,sig,mu,estComp)
+			if(i %% thin == 0){
+				for(j in 1:nfact)
+					Theta[[j]][,ind] <- theta0[,j]									
+				ind <- ind + 1
+			}			
+		}
+
+		expscores <- matrix(0,ncol=nfact,nrow=nrow(tabdata))
+		sdscores <- matrix(0,ncol=nfact,nrow=nrow(tabdata))
+		for(i in 1:nfact){
+			expscores[,i] <- rowMeans(Theta[[i]])
+			sdscores[,i] <- apply(Theta[[i]],1,sd)
+		}
+				
+		ret <- cbind(unique(data)[,1:length(K)],expscores,sdscores)
+		colnames(ret) <- Names
+		
+		if(!full.scores){ 
+			ret <- ret[order(expscores[,1]),]
+			rownames(ret) <- NULL
+			return(ret)
+		} else {
+			fulldata <- object@data
+			scoremat <- matrix(0,nrow=nrow(fulldata),ncol=nfact)
+			colnames(scoremat) <- paste("F",1:nfact,sep='')
+			tmp <- unique(data)[,1:length(K)]
+			for (j in 1:nrow(tabdata)){          
+				TFvec <- colSums(ifelse(t(fulldata) == tmp[j, ],1,0)) == ncol(fulldata)        
+				scoremat[TFvec, ] <- expscores[j, ]
+			}              
+			return(cbind(object@data,scoremat))
+		}	
+	}	
+)
+
+setMethod(
+	f = "fitted",
+	signature = signature(object = 'confmirtClass'),
+	definition = function(object, digits = 3, ...){  
+		tabdata <- object@tabdata
+		colnames(tabdata) <- c(colnames(object@data),"freq","exp")	
+		print(tabdata, digits)
+		invisible(tabdata)
+	}
+)
